/*
 * Copyright 2019-2021 Fairphone B.V.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * 		http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.fairphone.fp2.settings.app

import android.content.Context
import androidx.preference.PreferenceManager
import android.util.Log
import java.io.File
import java.io.IOException

object Settings {

    private const val BINARY_GETPROP = "/system/bin/getprop"
    private const val BINARY_SETPROP = "/system/bin/setprop"
    private const val EXPERIMENTAL_GLES3_PROPERTY_FILE = "/data/misc/fairphone/experimental_gles3"
    private const val ERROR_READING_GLES3_SETTING =
        "No choice stored regarding experimental OpenGL ES 3.0. Falling back to default (off)."
    private const val ERROR_WRITING_GLES3_SETTING = "Error while writing ExperimentalGLES3 file."
    private const val GYROSCOPE_PROPERTY = "persist.fp2.use_gyroscope"
    private const val ERROR_READING_GYROSCOPE_SETTING =
            "Error while regarding gyroscope property. Falling back to default (off)."
    private const val ERROR_WRITING_GYROSCOPE_SETTING =
            "Error while writing gyroscope property."
    private const val TAG = "FairphoneSettings"
    // Allow triggering the notification again when new settings are introduced.
    private const val CURRENT_NOTIFICATION_VERSION = 3
    private const val PREF_HAS_SHOWN_NOTIFICATION_VERSION = "PREF_HAS_SHOWN_NOTIFICATION_VERSION"

    private val openGLES3PropertyFile: File
        get() = File(EXPERIMENTAL_GLES3_PROPERTY_FILE)

    /**
     * Enable or disable experimental OpenGLES setting.
     *
     * @param enable:
     */
    fun enableOpenGLES30(enable: Boolean): Boolean {
        val openGLFlag = if (enable) "1" else "0"
        return try {
            openGLES3PropertyFile.writeText(openGLFlag)
            true
        } catch (e: IOException) {
            Log.e(
                TAG,
                ERROR_WRITING_GLES3_SETTING, e
            )
            false
        }
    }

    /**
     * @return true if experimental OpenGL ES 3.0 setting is enabled, false otherwise
     */
    fun isOpenGLES30Enabled(): Boolean {
        return try {
            openGLES3PropertyFile.readText().equals("1")
        } catch (e: Exception) {
            Log.e(
                TAG,
                ERROR_READING_GLES3_SETTING, e
            )
            false
        }
    }

    /**
     * Enable or disable experimental gyroscope
     *
     * @param enable:
     */
    fun enableGyroscope(enable: Boolean): Boolean {
        val flag = if (enable) "1" else "0"
        return try {
            setSystemProperty(GYROSCOPE_PROPERTY, flag)
            true
        } catch (e: Exception) {
            Log.e(
                TAG,
                ERROR_WRITING_GYROSCOPE_SETTING, e
            )
            false
        }
    }

    /**
     * @return true if experimental gyroscope is enabled, false otherwise
     */
    fun isGyroscopeEnabled(): Boolean {
        return try {
            getSystemProperty(GYROSCOPE_PROPERTY).equals("1")
        } catch (e: Exception) {
            Log.e(
                TAG,
                ERROR_READING_GYROSCOPE_SETTING, e
            )
            false
        }
    }

    fun hasShownNotification(context: Context): Boolean =
        getSharedPrefs(context).getInt(PREF_HAS_SHOWN_NOTIFICATION_VERSION, 0) ==
                CURRENT_NOTIFICATION_VERSION


    fun setHasShownNotification(context: Context) {
        val editor = getSharedPrefs(context).edit()
        editor.putInt(PREF_HAS_SHOWN_NOTIFICATION_VERSION, CURRENT_NOTIFICATION_VERSION)
        editor.apply()
    }

    private fun getSharedPrefs(context: Context) =
        PreferenceManager.getDefaultSharedPreferences(context)

    private fun getSystemProperty(key: String): String {
        val process = ProcessBuilder(BINARY_GETPROP, key)
                .redirectOutput(ProcessBuilder.Redirect.PIPE)
                .start()
        val retVal = process.waitFor()
        if (retVal != 0)
            throw Exception("`getprop $key` exited with failure $retVal")
        return process.inputStream.bufferedReader().readText().trim('\n')
    }

    private fun setSystemProperty(key: String, value: String) {
        val process = ProcessBuilder(BINARY_SETPROP, key, value).start()
        val retVal = process.waitFor()
        if (retVal != 0)
            throw Exception("`setprop $key $value` exited with failure $retVal")
    }

}